;==============================================================================
; Clock Control Logic
;------------------------------------------------------------------------------

; Notes:
;
; The 16F74 does not have any EEPROM so the memory area of the RTC is used to
; store alarm times.
;
; The RTC is forced into 12-Hour mode during setup. The alarm times are
; recorded as 12-Hour times in the same pattern used by the RTC chip so they
; can be directly compared.

;==============================================================================
; Revision History:
;
; 2005-02-16    Added alarm sounding and silencing states.
;
; 2005-02-06    Initial version
;------------------------------------------------------------------------------

                include ../pic.inc
                include ../lib/dallas1307.inc
                include ../lib/config.inc
                include display.inc
                include hardware.inc

                global  BeginClocking

                extern  DS1307Tx
                extern  DS1307Rx
                extern  HexLedPattern

                extern  I2C_REGISTER
                extern  I2C_REGCOUNT
                extern  SCRATCH

; The RTC memory registers used to hold alarm times.

RTC_AL_DAY_MIN  equ     h'10'
RTC_AL_DAY_HOUR equ     h'11'
RTC_AL_SAT_MIN  equ     h'12'
RTC_AL_SAT_HOUR equ     h'13'
RTC_AL_SUN_MIN  equ     h'14'
RTC_AL_SUN_HOUR equ     h'15'

; Define aliases for LED segments

SA              equ     SEGMENT_A
SB              equ     SEGMENT_B
SC              equ     SEGMENT_C
SD              equ     SEGMENT_D
SE              equ     SEGMENT_E
SF              equ     SEGMENT_F
SG              equ     SEGMENT_G

;==============================================================================
; Declare storage area for the current time and the alarm settings.

                udata
SECONDS         res     1
MINUTES         res     1
HOURS           res     1
DAY             res     1
DATE            res     1
MONTH           res     1
YEAR            res     1
CONTROL         res     1
CHANGED         res     1

AL_DAY_MIN      res     1
AL_DAY_HOUR     res     1
AL_SAT_MIN      res     1
AL_SAT_HOUR     res     1
AL_SUN_MIN      res     1
AL_SUN_HOUR     res     1

;==============================================================================

                code

; Read the alarm time settings from the RTC eeprom

BeginClocking:
                banksel I2C_REGISTER
                movlw   RTC_CONTROL
                movwf   I2C_REGISTER
                movlw   .1
                movwf   I2C_REGCOUNT

                bankisel SCRATCH
                movlw   CONTROL
                movwf   FSR
                movlw   h'10'
                movwf   CONTROL
                lcall   DS1307Tx

                banksel I2C_REGISTER
                movlw   RTC_AL_DAY_MIN
                movwf   I2C_REGISTER
                movlw   .6
                movwf   I2C_REGCOUNT

                bankisel AL_DAY_MIN
                movlw   AL_DAY_MIN
                movwf   FSR
                lcall   DS1307Rx

;------------------------------------------------------------------------------
; Most the time the clock is simply displaying the current time and waiting for
; either an alarm to be triggered or a button to be pressed

WaitForEvent:   call    UpdateTime
                call    TestForAlarm
                skpnc
                goto    AlarmTriggered
AlarmSilent:
                call    AlarmOff
                call    WaitForChange
                skpz
                call    HandleSwitches
                goto    WaitForEvent

;------------------------------------------------------------------------------
; If an alarm is triggered then flash the display and lamp while sounding the
; buzzer until a button is pressed.

WaitForQuiet:   call    UpdateTime
                call    TestForAlarm
                skpc
                goto    AlarmSilent
AlarmTriggered:
                call    AlarmPulse
                btfsc   CTRL,CTRL_BUZZ
                goto    AlarmSteady
                movlw   h'ff'
                movwf   SEG1
                movwf   SEG2
                movwf   SEG3
                movwf   SEG4
AlarmSteady:
                call    WaitForChange
                andlw   h'07'
                skpnz
                goto    WaitForQuiet
                call    WaitForRelease

;------------------------------------------------------------------------------
; Waits until the alarm period has been passed before returning to the normal
; wait loop.

WaitForUpdate:  call    UpdateTime
                call    TestForAlarm
                skpc
                goto    AlarmSilent
                call    AlarmOff

                call    WaitForChange
                skpz
                call    HandleSwitches
                goto    WaitForUpdate

;==============================================================================
; Wait until either one or more switches have been pressed or the 1Hz pulse has
; changed

WaitForChange:
                banksel PORTD           ; Read current 1Hz state
                movf    PORTD,W
                banksel SCRATCH
                movwf   SCRATCH         ; .. and store
ScanSwitches:
                banksel PORTE           ; Read the current state of the switch
                movf    PORTE,W         ; .. inputs
                call    WaitForSync     ; Wait for next interrupt
                xorwf   PORTE,W         ; Re-read switches and detect changes
                andlw   (1<<SW_R)|(1<<SW_H)|(1<<SW_M)
                skpz
                return                  ; Return if any changed

                banksel PORTD           ; Re-read the 1HZ input
                movf    PORTD,W 
                banksel SCRATCH
                xorwf   SCRATCH,W       ; Has it changed?
                andlw   (1<<HZ)
                skpnz
                goto    ScanSwitches    ; No, re-check the switches
                movlw   0
                return

;==============================================================================
; Returns with C=1 if the active alarm matches the current time.

TestForAlarm:
                call    GetAlarmFile
                movwf   FSR
                banksel MINUTES
                movf    MINUTES,W
                subwf   INDF,W
                skpz
                goto    ClearCarry
                incf    FSR,F
                movf    HOURS,W
                subwf   INDF,W
                skpz
                goto    ClearCarry

SetCarry:
                setc
                return

ClearCarry:
                clrc
                return

; Returns the register index corresponding to the alarm that should be used for
; today.

GetAlarmFile:
                bankisel AL_DAY_MIN
                banksel DAY
                movf    DAY,W
                addlw   -1
                skpnz
                retlw   AL_SUN_MIN
                addlw   -6
                skpnz
                retlw   AL_SAT_MIN
                retlw   AL_DAY_MIN

;==============================================================================
; If the 1Hz signal is high then sound the alarm else turn it off.

AlarmPulse:
                banksel PORTD
                btfsc   PORTD,HZ
                goto    AlarmOn

; Turn the control bits for the buzzer and lamp off so the next interrupt will
; silence the alarm.

AlarmOff:
                banksel CTRL
                bcf     CTRL,CTRL_BUZZ
                bcf     CTRL,CTRL_LAMP
                return

; Turn the control bits for the buzzer and lamp on so the next interrupt will
; sound the alarm.

AlarmOn:
                banksel CTRL
                bsf     CTRL,CTRL_BUZZ
                bsf     CTRL,CTRL_LAMP
                return

;==============================================================================
; Read the current time from the RTC and generate the values to update the
; display on the next interrupt.

UpdateTime:
                banksel I2C_REGISTER
                movlw   RTC_SECONDS
                movwf   I2C_REGISTER
                movlw   .7
                movwf   I2C_REGCOUNT

                bankisel SECONDS
                movlw   SECONDS
                movwf   FSR
                call    DS1307Rx

                bankisel MINUTES
                movlw   MINUTES
                movwf   FSR
                goto    ShowTime

;==============================================================================

HandleSwitches:
                banksel SCRATCH
                movwf   SCRATCH
                btfsc   SCRATCH,SW_R
                goto    AdjustTime
                btfsc   SCRATCH,SW_H
                goto    HandleDayDate    
                btfsc   SCRATCH,SW_M                
                goto    HandleMonthYear
                return

;------------------------------------------------------------------------------

AdjustTime:
                call    WaitForRelease
                banksel CHANGED         ; Clear the change value flag
                clrf    CHANGED
                banksel SECONDS         ; Reset the seconds
                clrf    SECONDS
                banksel CTRL            ; Turn on the lamp
                bsf     CTRL,CTRL_LAMP
TimeAdjust:
                bankisel MINUTES        ; Point the FSR at the MINUTES
                movlw   MINUTES
                movwf   FSR
                call    ShowTime        ; Update the displayed time
                banksel CTRL
                bcf     CTRL,CTRL_SECS
                call    WaitForChange
                banksel SCRATCH
                movwf   SCRATCH
                btfsc   SCRATCH,SW_R
                goto    TweakTime
                btfsc   SCRATCH,SW_H
                goto    TweakHour
                btfsc   SCRATCH,SW_M
                goto    TweakMinute
                goto    TimeAdjust

TweakHour:
                banksel HOURS
                incf    HOURS,W         ; Bump the hour by one
                andlw   h'3f'           ; Mask out all but the hours and PM
                addlw   .6              ; Add one and check for >9 digit
                skpdc
                addlw   -.6             ; Its not, so take back 6
                movwf   HOURS
                andlw   h'1f'           ; Mask out all but hours
                addlw   -h'13'          ; Is the value >12?
                skpc
                goto    Set12HourBit
                movf    HOURS,W
                addlw   .14
                movwf   HOURS
Set12HourBit:
                bsf     HOURS,6         ; Force AM/PM mode
                bsf     CHANGED,7       ; Set the changed flag
                call    WaitForRelease
                goto    TimeAdjust      ; Go and wait for another switch press

TweakMinute:
                banksel MINUTES
                incf    MINUTES,W       ; Bump the minutes by one
                addlw   .6              ; See if the lo nybble is >9
                skpdc
                addlw   -.6             ; Its not, so take the 6 back
                movwf   MINUTES         
                addlw   -h'60'          ; Is the value >59?
                skpnc
                clrf    MINUTES         ; Yes, set it back to 0
                bsf     CHANGED,7       ; Set the changed flag
                call    WaitForRelease
                goto    TimeAdjust      ; Go and wait for another switch press

TweakTime:
                banksel CHANGED         ; Was the time changed?
                btfss   CHANGED,7
                goto    AdjustDate

                banksel I2C_REGISTER    ; Set I2C registers to update SECONDS,
                movlw   RTC_SECONDS     ; .. MINUTES and HOURS
                movwf   I2C_REGISTER
                movlw   .3
                movwf   I2C_REGCOUNT
                bankisel SECONDS        ; Point the FSR at the SECONDS
                movlw   SECONDS
                movwf   FSR
                lcall   DS1307Tx        ; And write them back

;------------------------------------------------------------------------------

AdjustDate:
                call    WaitForRelease
                banksel CHANGED
                clrf    CHANGED

DateAdjust:
                call    ShowDate        ; Show current day and date                
                banksel CTRL
                bcf     CTRL,CTRL_SECS
                call    WaitForChange
                banksel SCRATCH
                movwf   SCRATCH
                btfsc   SCRATCH,SW_R
                goto    TweakDayDate
                btfsc   SCRATCH,SW_H
                goto    TweakDay
                btfsc   SCRATCH,SW_M
                goto    TweakDate
                goto    DateAdjust

TweakDay:
                banksel DAY             ; Bump the day by one
                incf    DAY,W
                andlw   h'07'           ; Mask back to a valid day
                skpnz
                movlw   .1              ; Ensure zero is skipped
                movwf   DAY
                bsf     CHANGED,7       ; Set the changed flag
                call    WaitForRelease
                goto    DateAdjust

TweakDate:
                banksel DATE
                movf    DATE,W          ; Wrap last day back to zero
                addlw   -h'31'
                skpnc
                clrf    DATE
                incf    DATE,W          ; Bump the date by one
                addlw   .6              ; Handle BCD carry
                skpdc
                addlw   -.6
                movwf   DATE
                bsf     CHANGED,7
                call    WaitForRelease
                goto    DateAdjust                

TweakDayDate:
                banksel CHANGED         ; Was the date changed?
                btfss   CHANGED,7
                goto    AdjustYear

                banksel I2C_REGISTER    ; Set I2C registers to update DAY
                movlw   RTC_DAY         ; .. and DATE
                movwf   I2C_REGISTER
                movlw   .2
                movwf   I2C_REGCOUNT
                bankisel DAY            ; Point the FSR at the SECONDS
                movlw   DAY
                movwf   FSR
                lcall   DS1307Tx        ; And write them back

;------------------------------------------------------------------------------

AdjustYear:
                call    WaitForRelease
                banksel CHANGED
                clrf    CHANGED

YearAdjust:
                call    ShowMonth       ; Show current month and year                
                banksel CTRL
                bcf     CTRL,CTRL_SECS
                call    WaitForChange
                banksel SCRATCH
                movwf   SCRATCH
                btfsc   SCRATCH,SW_R
                goto    TweakYearMonth
                btfsc   SCRATCH,SW_H
                goto    TweakMonth
                btfsc   SCRATCH,SW_M
                goto    TweakYear
                goto    YearAdjust

TweakMonth:
                banksel MONTH
                movf    MONTH,W
                addlw   -h'12'
                skpnc
                clrf    MONTH
                incf    MONTH,W
                addlw   .6
                skpdc
                addlw   -.6
                movwf   MONTH                
                bsf     CHANGED,7
                call    WaitForRelease
                goto    YearAdjust                

TweakYear:
                banksel YEAR
                incf    YEAR,W
                addlw   .6
                skpdc
                addlw   -.6
                addlw   h'60'
                skpc
                addlw   -h'60'
                movwf   YEAR
                bsf     CHANGED,7
                call    WaitForRelease
                goto    YearAdjust                

TweakYearMonth:
                banksel CHANGED
                btfss   CHANGED,7
                goto    AdjustAlarmDy

                banksel I2C_REGISTER    ; Set I2C registers to update MONTH
                movlw   RTC_MONTH       ; .. and YEAR
                movwf   I2C_REGISTER
                movlw   .2
                movwf   I2C_REGCOUNT
                bankisel MONTH          ; Point the FSR at the MONTH
                movlw   MONTH
                movwf   FSR
                lcall   DS1307Tx        ; And write them back

;------------------------------------------------------------------------------

AdjustAlarmDy:
                call    WaitForRelease
                banksel CHANGED
                clrf    CHANGED

AlarmDyAdjust:
                bankisel AL_DAY_MIN     ; Point the FSR at the MINUTES
                movlw   AL_DAY_MIN
                movwf   FSR
                call    ShowTime        ; Update the displayed time 
                banksel CTRL
                bcf     CTRL,CTRL_SECS
                call    WaitForChange
                banksel SCRATCH
                movwf   SCRATCH
                btfsc   SCRATCH,SW_R
                goto    TweakAlarmDy
                btfsc   SCRATCH,SW_H
                goto    TweakHourDy
                btfsc   SCRATCH,SW_M
                goto    TweakMinDy
                goto    AlarmDyAdjust

TweakHourDy:
                bankisel AL_DAY_HOUR
                movlw   AL_DAY_HOUR
                movwf   FSR
                call    TweakAlarmHour
                banksel CHANGED
                bsf     CHANGED,7
                call    WaitForRelease
                goto    AlarmDyAdjust

TweakMinDy:
                bankisel AL_DAY_MIN
                movlw   AL_DAY_MIN
                movwf   FSR
                call    TweakAlarmMinute
                banksel CHANGED
                bsf     CHANGED,7
                call    WaitForRelease
                goto    AlarmDyAdjust

TweakAlarmDy:
                banksel CHANGED
                btfss   CHANGED,7
                goto    AdjustAlarmSa

                banksel I2C_REGISTER    ; Set I2C registers to update MINUTE
                movlw   RTC_AL_DAY_MIN  ; .. and HOUR
                movwf   I2C_REGISTER
                movlw   .2
                movwf   I2C_REGCOUNT
                bankisel AL_DAY_MIN     ; Point the FSR at the MINUTE
                movlw   AL_DAY_MIN
                movwf   FSR
                lcall   DS1307Tx        ; And write them back

;------------------------------------------------------------------------------

AdjustAlarmSa:
                call    WaitForRelease
                banksel CHANGED
                clrf    CHANGED

AlarmSaAdjust:
                bankisel AL_SAT_MIN     ; Point the FSR at the MINUTES
                movlw   AL_SAT_MIN
                movwf   FSR
                call    ShowTime        ; Update the displayed time 
                banksel CTRL
                bcf     CTRL,CTRL_SECS
                call    WaitForChange
                banksel SCRATCH
                movwf   SCRATCH
                btfsc   SCRATCH,SW_R
                goto    TweakAlarmSa
                btfsc   SCRATCH,SW_H
                goto    TweakHourSa
                btfsc   SCRATCH,SW_M
                goto    TweakMinSa
                goto    AlarmSaAdjust

TweakHourSa:
                bankisel AL_SAT_HOUR
                movlw   AL_SAT_HOUR
                movwf   FSR
                call    TweakAlarmHour
                banksel CHANGED
                bsf     CHANGED,7
                call    WaitForRelease
                goto    AlarmSaAdjust

TweakMinSa:
                bankisel AL_SAT_MIN
                movlw   AL_SAT_MIN
                movwf   FSR
                call    TweakAlarmMinute
                banksel CHANGED
                bsf     CHANGED,7
                call    WaitForRelease
                goto    AlarmSaAdjust

TweakAlarmSa:
                banksel CHANGED
                btfss   CHANGED,7
                goto    AdjustAlarmSu

                banksel I2C_REGISTER    ; Set I2C registers to update MINUTE
                movlw   RTC_AL_SAT_MIN  ; .. and HOUR
                movwf   I2C_REGISTER
                movlw   .2
                movwf   I2C_REGCOUNT
                bankisel AL_SAT_MIN     ; Point the FSR at the MINUTE
                movlw   AL_SAT_MIN
                movwf   FSR
                lcall   DS1307Tx        ; And write them back

;------------------------------------------------------------------------------

AdjustAlarmSu:
                call    WaitForRelease
                banksel CHANGED
                clrf    CHANGED

AlarmSuAdjust:
                bankisel AL_SUN_MIN     ; Point the FSR at the MINUTES
                movlw   AL_SUN_MIN
                movwf   FSR
                call    ShowTime        ; Update the displayed time 
                banksel CTRL
                bcf     CTRL,CTRL_SECS
                call    WaitForChange
                banksel SCRATCH
                movwf   SCRATCH
                btfsc   SCRATCH,SW_R
                goto    TweakAlarmSu
                btfsc   SCRATCH,SW_H
                goto    TweakHourSu
                btfsc   SCRATCH,SW_M
                goto    TweakMinSu
                goto    AlarmSuAdjust

TweakHourSu:
                bankisel AL_SUN_HOUR
                movlw   AL_SUN_HOUR
                movwf   FSR
                call    TweakAlarmHour
                banksel CHANGED
                bsf     CHANGED,7
                call    WaitForRelease
                goto    AlarmSuAdjust

TweakMinSu:
                bankisel AL_SUN_MIN
                movlw   AL_SUN_MIN
                movwf   FSR
                call    TweakAlarmMinute
                banksel CHANGED
                bsf     CHANGED,7
                call    WaitForRelease
                goto    AlarmSuAdjust

TweakAlarmSu:
                banksel CHANGED
                btfss   CHANGED,7
                goto    AdjustmentOver

                banksel I2C_REGISTER    ; Set I2C registers to update MINUTE
                movlw   RTC_AL_SUN_MIN  ; .. and HOUR
                movwf   I2C_REGISTER
                movlw   .2
                movwf   I2C_REGCOUNT
                bankisel AL_SUN_MIN     ; Point the FSR at the MINUTE
                movlw   AL_SUN_MIN
                movwf   FSR
                lcall   DS1307Tx        ; And write them back

AdjustmentOver:
                call    WaitForRelease
                banksel CTRL
                bcf     CTRL,CTRL_LAMP
                return

;------------------------------------------------------------------------------

TweakAlarmHour:
                incf    INDF,W          ; Bump the hour by one
                andlw   h'3f'           ; Mask out all but the hours and PM
                addlw   .6              ; Add one and check for >9 digit
                skpdc
                addlw   -.6             ; Its not, so take back 6
                movwf   INDF
                andlw   h'1f'           ; Mask out all but hours
                addlw   -h'13'          ; Is the value >12?
                skpc
                goto    Set12HourBitAl
                movf    INDF,W
                addlw   .14
                movwf   INDF
Set12HourBitAl:
                bsf     INDF,6          ; Force AM/PM mode
                return

TweakAlarmMinute:
                incf    INDF,W          ; Bump the minutes by one
                addlw   .6              ; See if the lo nybble is >9
                skpdc
                addlw   -.6             ; Its not, so take the 6 back
                movwf   INDF         
                addlw   -h'60'          ; Is the value >59?
                skpnc
                clrf    INDF            ; Yes, set it back to 0
                return
                    
;------------------------------------------------------------------------------

HandleDayDate:
                call    ShowDate
                goto    WaitForRelease

;------------------------------------------------------------------------------

HandleMonthYear:
                call    ShowMonth

;------------------------------------------------------------------------------

WaitForRelease:
                call    WaitForSync     ; Give the switches time to settle
                banksel PORTE           ; Read the switch states
                movf    PORTE,W
                xorlw   h'ff'           ; Invert the switch states
                andlw   (1<<SW_R)|(1<<SW_H)|(1<<SW_M)
                skpz
                goto    WaitForRelease  ; Repeat until all switches released
                return

;------------------------------------------------------------------------------

ShowTime:

; Converts minutes to led pattern

                swapf   INDF,W
                andlw   h'0f'
                lcall   HexLedPattern
                xorlw   h'ff'
                banksel SEG3
                movwf   SEG3
                movf    INDF,W
                andlw   h'0f'
                lcall   HexLedPattern
                xorlw   h'ff'
                banksel SEG4
                movwf   SEG4
                incf    FSR,F

; Converts hours to led pattern

                swapf   INDF,W
                andlw   h'01'
                lcall   HexLedPattern
                xorlw   h'ff'
                banksel SEG1
                movwf   SEG1
                movf    INDF,W
                andlw   h'0f'
                lcall   HexLedPattern
                xorlw   h'ff'
                banksel SEG2
                movwf   SEG2

; Set AM/PM indicator

                movf    INDF,W
                banksel CTRL
                bcf     CTRL,CTRL_AMPM
                andlw   h'20'
                skpz
                bsf     CTRL,CTRL_AMPM

; Use 1Hz input to set seconds indicator

                banksel PORTD
                movf    PORTD,W
                banksel CTRL
                bcf     CTRL,CTRL_SECS
                andlw   (1<<HZ)
                skpz
                bsf     CTRL,CTRL_SECS

                return

;------------------------------------------------------------------------------
; Changes the display to show the current day and date

ShowDate:
                bankisel DAY            ; Point FSR at DAY
                movlw   DAY
                movwf   FSR

                movf    INDF,W          ; Determine the LED pattern for the
                andlw   .7              ; .. first letter
                call    LetterPattern
                xorlw   h'ff'           ; Invert to match hardware
                banksel SEG1            ; And store in SEG1 pattern buffer
                movwf   SEG1
                movf    INDF,W          ; Determine the LED pattern for the
                andlw   .7              ; .. second letter
                iorlw   .8
                call    LetterPattern
                xorlw   h'ff'           ; Invert to match hardware
                banksel SEG2            ; And store in SEG2 pattern buffer
                movwf   SEG2

                incf    FSR,F           ; Point FSR at DATE

                swapf   INDF,W          ; Convert date hi nybble to LED
                andlw   h'0f'           ; .. pattern
                lcall   HexLedPattern
                xorlw   h'ff'           ; Invert to match hardware
                banksel SEG3            ; And store in SEG3 pattern buffer
                movwf   SEG3
                movf    INDF,W          ; Convert date lo nybble to LED
                andlw   h'0f'           ; .. pattern
                call    HexLedPattern
                xorlw   h'ff'           ; Invert to match hardware
                banksel SEG4            ; And store in SEG4 pattern buffer
                movwf   SEG4
                bcf     CTRL,CTRL_AMPM  ; Ensure AM/PM LED is off
                return

LetterPattern:
                banksel SCRATCH         ; Save day code in scratch area
                movwf   SCRATCH
                movlw   high Values     ; Set up PCLATH for jump table
                movwf   PCLATH
                movlw   low Values      ; Compute address of table entry
                addwf   SCRATCH,W
                skpnc                   ; Increment PCLATH if jump goes across
                incf    PCLATH,F        ; .. page boundary.
                movwf   PCL             ; Goto the target entry

Values          retlw                     SG    ; --
                retlw   SA   |SC|SD|   SF|SG    ; Su
                retlw   SA|SB|SC|   SE|SF       ; Mo
                retlw   SA|SB|SC                ; Tu
                retlw      SB|SC|SD|SE|SF       ; We
                retlw   SA|SB|SC                ; Th
                retlw   SA|         SE|SF|SG    ; Fr
                retlw   SA   |SC|SD|   SF|SG    ; Sa

                retlw                     SG    ; --
                retlw         SC|SD|SE          ; Su
                retlw         SC|SD|SE|   SG    ; Mo
                retlw         SC|SD|SE          ; Tu
                retlw   SA|      SD|SE|SF|SG    ; We
                retlw         SC|   SE|SF|SG    ; Th
                retlw               SE|   SG    ; Fr
                retlw   SA|SB|SC|   SE|SF|SG    ; Sa

;------------------------------------------------------------------------------
; Changes the display to show the current month and year

ShowMonth:
                bankisel MONTH          ; Point FSR at the MONTH
                movlw   MONTH
                movwf   FSR

                swapf   INDF,W          ; Convert month hi nybble to LED
                andlw   h'0f'           ; .. pattern
                lcall   HexLedPattern
                xorlw   h'ff'           ; Invert to match hardware
                banksel SEG1            ; And store in SEG1 pattern buffer
                movwf   SEG1    
                movf    INDF,W          ; Convert month lo nybble to LED
                andlw   h'0f'           ; .. pattern
                lcall   HexLedPattern
                xorlw   h'ff'           ; Invert to match hardware
                banksel SEG2            ; And store in SEG2 pattern buffer
                movwf   SEG2                                

                incf    FSR,F           ; Point FSR at the YEAR

                swapf   INDF,W          ; Convert month hi nybble to LED
                andlw   h'0f'           ; .. pattern
                lcall   HexLedPattern
                xorlw   h'ff'           ; Invert to match hardware
                banksel SEG3            ; And store in SEG1 pattern buffer
                movwf   SEG3    
                movf    INDF,W          ; Convert month lo nybble to LED
                andlw   h'0f'           ; .. pattern
                lcall   HexLedPattern
                xorlw   h'ff'           ; Invert to match hardware
                banksel SEG4            ; And store in SEG2 pattern buffer
                movwf   SEG4                                
                bcf     CTRL,CTRL_AMPM  ; Ensure AM/PM LED is off
                return

;==============================================================================
; Wait until the interrupt routine clears the synchronisation flag.

WaitForSync:
                banksel CTRL            ; Set the synchronisation bit
                bsf     CTRL,CTRL_SYNC
                btfsc   CTRL,CTRL_SYNC  ; Then wait until it is cleared
                goto    $-1
                return

                end
